/** 
 *	TrafficControlServerImpl.java
 *	
 *	Created by Dehua Zhang
 *	Modified by Dehua Zhang
 *	Modified by Greg Bylenok
 *	Modified by Deepti Srivastava
 */
 
/**
 *	Modification Log
 *
 *	This implementation version uses a hashtable for storing information about planes in the airspace
 *	and to map an ID to the information about a particular plane. The server gets the next ID to be assigned from 
 *	the database
 */	

package Server;

import ServerModule.TrafficControlPOA;
import ServerModule.invalid_coordinate;
import ServerModule.PlanePosition_Detail;
import org.omg.CORBA.ORB;
import Server.db.IDBManager;
import Server.db.DBFactory;

import java.util.*;
import java.sql.SQLException;

class TrafficControlServerImpl extends TrafficControlPOA {

    private ORB orb = null;
    private Map map = null;
    private boolean mDebugMode = false;

	private Hashtable planesList = new Hashtable();

    private IDBManager db = null;

    public void Init(ORB orb_val, boolean debugMode) throws SQLException {
        orb = orb_val;
        map = new Map();
        this.mDebugMode = debugMode;
        db = DBFactory.getManager();
        db.setDebug(mDebugMode);
    }

    /**
     *  Clients: Planes
     */
    public String PositionUpdate(int id, float x, float y, float vx, float vy)
            throws ServerModule.invalid_plane_id, ServerModule.invalid_coordinate {
            

        //check for valid coordinates
        String towerName = null;
        towerName = map.lookup(x, y);
        if (towerName == null)   throw new invalid_coordinate();

        //retrieve plane from memory
        Integer id_object=new Integer(id);
        Plane _plane = (Plane)this.planesList.get(id_object);
        if (_plane == null) throw new ServerModule.invalid_plane_id();

        //save this to database
        try {
            db.savePosition(new Plane(id, x, y, vx, vy));
        } catch (SQLException e) {
                traceError("Error saving client position to database.", e);
                //TODO: create generic application exception to pass to client. For now we have to eat it.
        }

        //save in memory
        _plane.setX(x);
        _plane.setY(y);
        _plane.setVx(vx);
        _plane.setVy(vy);

	System.out.println("Position updated for Client "+id+"\n");

        return towerName;
    }

    /**
     * Clients: Planes
     * implementation of join using hashable
     */
    public int Join(String name) throws ServerModule.system_overload {

        int UID = -1; //this is a place holder for the UID got from the
                      //database to be assigned to a plane that joins

        //save in database
        try {
            UID = db.joinPlane(name);
        } catch (SQLException e) {
            traceError("Error getting UID from database", e);
            //TODO: create generic application exception to pass to client.
        }

        //save in memory
        Integer key = new Integer(UID);
        Plane newPlane = new Plane(UID);
        planesList.put(key, newPlane);

	System.out.println("Client " +name+ " sucessfully joined with UID " +UID+ "\n");

        return UID;
    }

    /**
     *  Clients: Planes
     */
    public boolean Leave(int id) throws ServerModule.invalid_plane_id {

        //retrieve plane from memory
        Integer id_object=new Integer(id);
        Plane _plane = (Plane)this.planesList.get(id_object);
        if (_plane == null) throw new ServerModule.invalid_plane_id();

        //remove plane from database
		try {
            db.removePlane(id);
        } catch (SQLException e) {
            traceError("Error removing plane from database", e);
            return false;
        }

        //remove plane from memory
        _plane =(Plane) (planesList.remove(id_object));
	
	System.out.println("Client with UID " +id+ " has successfully left \n");
	
	return true;

    }

    /**
     *  Clients: Planes
     */
    public ServerModule.PlanePosition_Brief[] Inquiry(int id, float x, float y, float radius1)
            throws ServerModule.invalid_plane_id, ServerModule.invalid_coordinate, ServerModule.system_overload {
            
        //check if the ID is valid
        Integer id_object=new Integer(id);
        Plane clientPlane = (Plane)this.planesList.get(id_object);
        if (clientPlane == null) throw new ServerModule.invalid_plane_id();
            
        //find the distance between the given plane and other planes by distance formula
        // and then see if it is less than or equal to radius1
        //We return the array containing the ids of all the planes within that radius
        ArrayList _result = new ArrayList();
        double x1;
        double y1;
        double distance = 0.0;
        Enumeration e = planesList.elements();
        
        while (e.hasMoreElements())  {
		
			Plane currentPlane = (Plane)e.nextElement();

            int i = currentPlane.getId();
            boolean active = currentPlane.isActive();
            if ((i != id) && (active == true)) //don't return the info of the plane itself or if plane is not active
            {
                x1 = currentPlane.getX();
                y1 = currentPlane.getY();
                distance = Math.sqrt(Math.pow(x - x1, 2.0) + Math.pow(y - y1, 2.0));
                
                if (distance < radius1) {
                    _result.add(new ServerModule.PlanePosition_Brief
                            (currentPlane.getId(), currentPlane.getX(), currentPlane.getY()));
               }             
            }
        }
        int returun_array_size = _result.size();
        
        ServerModule.PlanePosition_Brief[] returnArray = new ServerModule.PlanePosition_Brief[returun_array_size];
        //copy the value into the return array
        for (int i = 0; i < returun_array_size; i++) {
            returnArray[i] = (ServerModule.PlanePosition_Brief) _result.get(i);
        }
        
        System.out.println("Serviced Inquiry made by Client " +id+"\n");
        
        return returnArray;
    }

    /**
     *  Clients: Planes
     */
    public ServerModule.PlanePosition_Detail[] EmergencyInquiry(int id, float x,
                                                                float y, float radius2) throws ServerModule.invalid_plane_id, ServerModule.invalid_coordinate {
        //check if the ID is valid
        Integer id_object=new Integer(id);
        Plane clientPlane = (Plane)this.planesList.get(id_object);
        if (clientPlane == null) throw new ServerModule.invalid_plane_id();

        //find the distance between the given plane and other planes by distance formula
        // and then see if it is less than or equal to radius1
        //We return the array containing the ids of all the planes within that radius
        ArrayList _result = new ArrayList();
        double x1;
        double y1;
        double distance = 0.0;
        Enumeration e = planesList.elements() ;
        
        while (e.hasMoreElements())  {
		
			Plane currentPlane = (Plane)e.nextElement();

            int i = currentPlane.getId();
            boolean active = currentPlane.isActive();
            if ((i != id) && (active == true)) //don't return the info of the plane itself or if plane is not active
            {
                x1 = currentPlane.getX();
                y1 = currentPlane.getY();
                distance = Math.sqrt(Math.pow(x - x1, 2.0) + Math.pow(y - y1, 2.0));
                if (distance < radius2) {
                    PlanePosition_Detail _object = new ServerModule.PlanePosition_Detail();
                    _object.id = currentPlane.getId();
                    _object.x = currentPlane.getX();
                    _object.y = currentPlane.getY();
                    _object.vx = currentPlane.getVx();
                    _object.vy = currentPlane.getVy();
                    
                    _result.add(_object);
                }
            }
        }
        int returun_array_size = _result.size();
        ServerModule.PlanePosition_Detail[] returnArray = new ServerModule.PlanePosition_Detail[returun_array_size];
       
        //copy the value into the return array
        for (int i = 0; i < returun_array_size; i++) {
            returnArray[i] = (ServerModule.PlanePosition_Detail) _result.get(i);
        }
        
        System.out.println("Serviced Emergency Inquiry made by Client " +id+"\n");
        
        return returnArray;
    }

    // implement shutdown() method
    public void shutdown() {
        orb.shutdown(false);
    }
  
    public boolean is_alive() {
      return true;
    }

    public boolean become_primary() {
        System.out.println("Server will be the primary\n");
        //Load the states from database
        planesList.clear();

        try {
            Plane[] planes = db.loadPositions();
            for (int i = 0; i < planes.length; i++) {
                Plane plane = planes[i];

                Integer idObject = new Integer(plane.getId());
                planesList.put(idObject, plane);
            }
        } catch (SQLException e) {
            traceError("Error retrieving state from database", e);
            return false;
            //TODO: maybe create generic application exception to pass to client.
        }
        System.out.println("Server has become the primary\n");
        return true;
    }

    /**
     * Prints debug info.
     */
    private void trace(String message) {
        if (mDebugMode)
            System.out.println(message);
    }

    /**
     * Prints exception message. Only prints stack trace if debug mode is on.
     */
    private void traceError(String message, Exception e) {

        System.err.println(message);
        if (mDebugMode)
            e.printStackTrace();
    }
}
