package Server.db;

import Server.ITower;
import Server.Plane;

import java.util.Properties;
import java.io.PrintWriter;
import java.io.InputStream;
import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.DriverManager;

public class DBManager implements IDBManager {

    private static final String PROPS_FILE = "server.properties";

    private static final int DEFAULT_TIMEOUT = 10; // seconds
    private String mURL = "";
    private String mUser = "";
    private String mPassword = "";
    private String mDriverClass = "";
    private boolean mDebugMode = false;

    public DBManager() {

        //defaults for now
        //PrintWriter log = new PrintWriter(System.out);
        //DriverManager.setLogWriter(log);
        //DriverManager.setLoginTimeout(DEFAULT_TIMEOUT);

        initFromProperties();

        //load the driver
        try {
            this.getClass().forName(mDriverClass);
        } catch (ClassNotFoundException e) {
            traceError("Could not load DB driver class '" + mDriverClass + "'",e);
        }

    }

    /**
     * Loads properties from properties files
     */
    private void initFromProperties() {
        //load properties
        InputStream propsStream = this.getClass().getClassLoader().getResourceAsStream(PROPS_FILE);

        if (propsStream == null) {
            trace("DBManager could not find properties.");
        }
        Properties props = new Properties();

        try {
            props.load(propsStream);
        } catch (IOException e) {
            traceError("DBManager could not load properties.",e);
        }

        mURL = (String) props.get("database.url");
        mUser = (String) props.get("database.user");
        mPassword = (String) props.get("database.password");
        mDriverClass = (String) props.get("database.driver");
    }

    private Connection getConnection() throws SQLException {

        Connection conn = null;
        try {
            conn = DriverManager.getConnection(mURL, mUser, mPassword);
        } catch (SQLException e) {
            traceError("Could not establish connection to datasource.",e);
            throw e;
        }

        if (conn == null)
        {
            trace("Could not establish connection to datasource.");
            throw new SQLException("Could not establish connection to datasource.");
        }
        return conn;
    }
/* saving positions of airplane*/
    public void savePosition(Plane aPosition) throws SQLException {
        Connection conn = getConnection();
        try {
            DBPlane.updatePlane(conn, aPosition);
        } catch (SQLException se) {
            traceError("DBManager could not save positions to DB.",se);
            throw se;
        } finally {
            closeConnection(conn);
        }
        return;
    }


    /**
     * Loads towers from database
     */
    public ITower[] loadTowers() throws SQLException {

        DBTower[] toReturn = null;
        Connection conn = getConnection();
        try {
            toReturn = DBTower.load(conn);
        } catch (SQLException se) {
            traceError("DBManager could not load from DB.",se);
            throw se;
        } finally {
            closeConnection(conn);
        }
        return toReturn;
    }

    /**
     *  Loads plane positions from database
     */
    public Plane[] loadPositions() throws SQLException {

        Plane[] toReturn = null;
        Connection conn = getConnection();
        try {

            toReturn = DBPlane.loadAll(conn);

        } catch (SQLException se) {
            traceError("DBManager could not read planes from DB.",se);
            throw se;
        } finally {
            closeConnection(conn);
        }
        return toReturn;
    }

    /**
     * Basically, inserts a plane into the database.
     *
     * @param planeName
     * @return UID of plane
     * @throws SQLException
     */
    public int joinPlane(String planeName) throws SQLException {

        int toReturn = -1;
        Connection conn = getConnection();
        try {

            toReturn = DBPlane.insertPlane(conn,planeName);

        } catch (SQLException se) {
            traceError("DBManager could not insert plane into DB.",se);
            throw se;
        } finally {
            closeConnection(conn);
        }
        return toReturn;
    }

    /**
     * Basically, removes a plane from the database
     * @param uid of plane to remove
     * @throws SQLException
     */
    public void removePlane(int uid) throws SQLException {
        Connection conn = getConnection();
        try {

            DBPlane.removePlane(conn,uid);

        } catch (SQLException se) {
            traceError("DBManager could not remove plane from DB.",se);
            throw se;
        } finally {
            closeConnection(conn);
        }
    }

    /**
     * Closes connection to the database. Make sure to call this after performing an operation!
     *
     * @param conn
     * @throws SQLException
     */
    private void closeConnection(Connection conn) throws SQLException {
            if (conn != null) {
                try {
                    conn.close(); //the connection is closed between every execution. This
                } catch (SQLException e) {
                    traceError("Warning: DBManager could not close connection.",e);
                    throw e;
                }
            }
        }


    public void setDebug(boolean debugMode) {
        mDebugMode = debugMode;
    }

    /**
     * Prints debug info.
     */
    private void trace(String message) {
        if (mDebugMode)
            System.out.println(message);
    }

    /**
     * Prints exception message. Only prints stack trace if debug mode is on.
     */
    private void traceError(String message, Exception e) {

        System.err.println(message);
        if (mDebugMode)
            e.printStackTrace();
    }


}
