package Server.test;

import bp.MultiThreadedTestCase;
import org.omg.CORBA.ORB;
import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContextExtHelper;
import ServerModule.TrafficControlHelper;
import ServerModule.TrafficControl;
import ServerModule.PlanePosition_Brief;

import java.util.Random;



public class TestInquiry extends MultiThreadedTestCase {

    public static int maxClients = 10;

    /**
     * Basic constructor - called by the test runners.
     */
    public TestInquiry(String s) {
        super(s);
    }

    public void test1() {
        TestCaseRunnable tct [] = new TestCaseRunnable[maxClients];
        TestDumbClientThread dct[] = new TestDumbClientThread[maxClients -1];

        Object monitor = new Object(); // this is just an object that is shared by all the threads.
                                       // The dumb clients will wait on this monitor.
                                       // The inquiry client will then notify when it is through with its tests.

        //add a number of dumb clients
        Random random = new Random();
        for (int i = 0; i < tct.length -1; i++) {
            dct[i] = new TestDumbClientThread(monitor, random);
            tct[i] = dct[i];
        }

        //add one client to test the inquiry function
        tct[tct.length-1] = new TestInquiryClientThread(monitor, dct);

        //kick off all threads
        runTestCaseRunnables(tct);
    }

    /**
     * This thread represents a plane that will just stand in place.
     */
    protected class TestDumbClientThread extends MultiThreadedTestCase.TestCaseRunnable {

        Object monitor = null;

        TrafficControl tfImpl;
        private float x;    //some x position for this plane
        private float y;    //some y position for this plane

        /**
         * @param monitor - just some object to synchronize on.
         */
        public TestDumbClientThread(Object monitor, Random random) {
            this.monitor = monitor;

            this.x = random.nextFloat();
            this.y = random.nextFloat();

        }

        /**
         * @return x position of plane
         */
        public float getX() {
            return x;
        }
        /**
         * @return y position of plane
         */
        public float getY() {
            return y;
        }

        /**
         * Sets up the CORBA stuff
         * @throws Exception - any exception
         */
        public void setupORB() throws Exception {
            // create and initialize the ORB
            String[] args = new String[]{"-ORBInitialPort", "1500"};
            ORB orb = ORB.init(args, null);

            // get the root naming context
            org.omg.CORBA.Object objRef = orb.resolve_initial_references("NameService");

            // Use NamingContextExt instead of NamingContext. This is
            // part of the Interoperable naming Service.
            NamingContextExt ncRef = NamingContextExtHelper.narrow(objRef);

            // resolve the Object Reference in Naming
            String name = "TrafficControlServer";
            tfImpl = TrafficControlHelper.narrow(ncRef.resolve_str(name));

        }


        /**
         * This is a relatively simple test that just calls positionUpdate
         * over and over again. The planes don't even move.
         *
         * @throws Throwable
         */
        public void runTestCase() throws Throwable {

            setupORB();

            int planeID = 0;

            planeID = tfImpl.Join("plane_test");


            tfImpl.PositionUpdate(planeID,x,y,0f, 0f);

            System.out.println("Register plane at ("+x+","+y+")");

            //go to sleep until the TestInquiryClientThread is through.
            synchronized (monitor)
            {
                monitor.wait();
            }

            tfImpl.Leave(planeID);

        }

    }


    /**
     * This thread represents a plane that tests for other planes in the airspace
     */
    protected class TestInquiryClientThread extends MultiThreadedTestCase.TestCaseRunnable {

        TrafficControl tfImpl;

        Object monitor = null;
        TestDumbClientThread dtc[] = null; //references ot the dumb clients.
        boolean foundDTC[] = null; //an array of flags that signal whether this client has located each dumb client

        /**
         * @param monitor - object to synchronize on
         * @param dtc - array of dumb client threads. This thread will search for each dumb client thread in the server's airspace
         */
        public TestInquiryClientThread(Object monitor, TestDumbClientThread dtc[]) {
            super();
            this.monitor = monitor;
            this.dtc = dtc;
            this.foundDTC = new boolean[dtc.length];
            for (int i = 0; i < foundDTC.length; i++) {
                foundDTC[i] = false;
            }
        }

        /**
         * Sets up the CORBA stuff
         * @throws Exception - any exception
         */
        public void setupORB() throws Exception {
            // create and initialize the ORB
            String[] args = new String[]{"-ORBInitialPort", "1500"};
            ORB orb = ORB.init(args, null);

            // get the root naming context
            org.omg.CORBA.Object objRef = orb.resolve_initial_references("NameService");

            // Use NamingContextExt instead of NamingContext. This is
            // part of the Interoperable naming Service.
            NamingContextExt ncRef = NamingContextExtHelper.narrow(objRef);

            // resolve the Object Reference in Naming
            String name = "TrafficControlServer";
            tfImpl = TrafficControlHelper.narrow(ncRef.resolve_str(name));

        }


        /**
         * @throws Throwable
         */
        public void runTestCase() throws Throwable {

            setupORB();

            int planeID = 0;

            planeID = tfImpl.Join("plane_test");


            int counter = 0;
            int maxRetries = 10;
            do {
                Thread.currentThread().sleep(1000);
                PlanePosition_Brief[] planes = tfImpl.Inquiry(planeID,0.5f,0.5f,0.71f); //radius of .71 should cover the whole airspace
                locatePlane(planes);
                counter ++;
            } while (!allLocated() && counter < maxRetries);

            synchronized (monitor)
            {
                //we are done with the test. Wake all the dumb clients so they leave the airspace.
                monitor.notifyAll();
            }

            //this client now leaves
            tfImpl.Leave(planeID);

            if (allLocated() == false)
            {
                //we should have located all the planes. It appears we timed out.
                MultiThreadedTestCase.fail("TestInquiryClient did not locate all planes in time.");
            }

        }

        /**
         * Checks to see if the plane position corresponds to any of
         * the dumb clients. If so, then flags it as located.
         *
         * @param planes Position of planes in the airspace
         */
        private void locatePlane(PlanePosition_Brief[] planes) {
            for (int i = 0; i < planes.length; i++) {
                PlanePosition_Brief plane = planes[i];
                for (int j = 0; j < dtc.length; j++) {

                    TestDumbClientThread dumbClientThread = dtc[j];
                    float dtcX = dumbClientThread.getX();
                    float dtcY = dumbClientThread.getY();

                    if ((foundDTC[j] == false) && (dtcX == plane.x) && (dtcY == plane.y))
                    {
                        System.out.println("Found plane "+j);
                        foundDTC[j] = true;
                    }

                }
            }

        }
           /**
            *
            * @return true if all planes are marked located.
            */
            private boolean allLocated() {
                boolean toReturn = true;
               for (int i = 0; i < foundDTC.length; i++) {
                   toReturn = toReturn & foundDTC[i];
               }
                return toReturn;
            }
    }

}